@@ -115,6 +115,8 @@ string HipBinNvidia::getHipCC() const {
   hipCCPath = cudaPath;
   hipCCPath /= "bin/nvcc";
   hipCC = hipCCPath.string();
+  if (getOSInfo() == windows)
+    hipCC = "\"" + hipCC + "\"";
   return hipCC;
 }
 
@@ -179,7 +181,11 @@ string HipBinNvidia::getHipInclude() const {
 void HipBinNvidia::initializeHipLdFlags() {
   string hipLdFlags;
   const string& cudaPath = getCompilerPath();
-  hipLdFlags = " -Wno-deprecated-gpu-targets -lcuda -lcudart -L" +
+  if (getOSInfo() == windows)
+    hipLdFlags = " -Wno-deprecated-gpu-targets -lcuda -lcudart -L\"" +
+               cudaPath + "/lib64\"";
+  else
+    hipLdFlags = " -Wno-deprecated-gpu-targets -lcuda -lcudart -L" +
                cudaPath + "/lib64";
   hipLdFlags_ = hipLdFlags;
 }
@@ -199,7 +205,10 @@ const string& HipBinNvidia::getHipLdFlags() const {
 void HipBinNvidia::initializeHipCFlags() {
   string hipCFlags;
   const string& cudaPath = getCompilerPath();
-  hipCFlags += " -isystem " + cudaPath + "/include";
+  if (getOSInfo() == windows)
+    hipCFlags += " -isystem \"" + cudaPath + "/include\"";
+  else
+    hipCFlags += " -isystem " + cudaPath + "/include";
   string hipIncludePath;
   hipIncludePath = getHipInclude();
   hipCFlags += " -isystem \"" + hipIncludePath + "\"";
@@ -215,7 +224,10 @@ const string& HipBinNvidia::getHipCXXFlags() const {
 void HipBinNvidia::initializeHipCXXFlags() {
   string hipCXXFlags = " -Wno-deprecated-gpu-targets ";
   const string& cudaPath = getCompilerPath();
-  hipCXXFlags += " -isystem " + cudaPath + "/include";
+  if (getOSInfo() == windows)
+    hipCXXFlags += " -isystem \"" + cudaPath + "/include\"";
+  else
+    hipCXXFlags += " -isystem " + cudaPath + "/include";
   string hipIncludePath;
   hipIncludePath = getHipInclude();
   hipCXXFlags += " -isystem \"" + hipIncludePath + "\"";
@@ -256,7 +268,10 @@ void HipBinNvidia::printCompilerInfo() const {
   fs::path nvcc;
   nvcc = getCompilerPath();
   nvcc /= "bin/nvcc";
-  cmd = nvcc.string() + " --version";
+  if (getOSInfo() == windows)
+    cmd = "\"" + nvcc.string() + "\"" + " --version";
+  else
+    cmd = nvcc.string() + " --version";
   system(cmd.c_str());
 }
 
@@ -266,7 +281,10 @@ string HipBinNvidia::getCompilerVersion() {
   fs::path nvcc;
   nvcc = getCompilerPath();
   nvcc /= "bin/nvcc";
-  cmd = nvcc.string() + " --version";
+  if (getOSInfo() == windows)
+    cmd = "\"" + nvcc.string() + "\"" + " --version";
+  else
+    cmd = nvcc.string() + " --version";
   system(cmd.c_str());
   return compilerVersion;
 }
@@ -378,7 +396,10 @@ void HipBinNvidia::executeHipCCCmd(vector<string> argv) {
     for (unsigned int i = 2; i < argv.size(); i++) {
       string isaarg = argv.at(i);
       ISACMD += " ";
-      if (!hipBinUtilPtr_->substringPresent(isaarg,"--rocm-path=")) {
+      if (hipBinUtilPtr_->substringPresent(isaarg,"--rocm-path=") ||
+          hipBinUtilPtr_->substringPresent(isaarg,"--hip-path=")) {
+        ISACMD += "-I" + hipBinUtilPtr_->splitStr(isaarg, '=')[1] + "/include";
+      } else {
         ISACMD += isaarg;
       }
     }
@@ -399,6 +420,12 @@ void HipBinNvidia::executeHipCCCmd(vector<string> argv) {
     string trimarg = hipBinUtilPtr_->replaceRegex(arg, toRemove, "");
     bool swallowArg = false;
     bool escapeArg = true;
+    // do not pass amd paths to nvcc
+    if (hipBinUtilPtr_->substringPresent(arg,"--rocm-path=") ||
+        hipBinUtilPtr_->substringPresent(arg,"--hip-path=")) {
+      continue;
+    }
+
     if (arg == "-c" || arg == "--genco" || arg == "-E") {
       compileOnly = true;
       needLDFLAGS  = false;
@@ -594,6 +621,9 @@ void HipBinNvidia::executeHipCCCmd(vector<string> argv) {
   }
   if (runCmd) {
     SystemCmdOut sysOut;
+    if (os == windows)
+      CMD = "\"" + CMD + "\"";
+
     sysOut = hipBinUtilPtr_->exec(CMD.c_str(), true);
     string cmdOut = sysOut.out;
     int CMD_EXIT_CODE = sysOut.exitCode;
