@@ -30,8 +30,6 @@ const char *Action::getClassName(ActionClass AC) {
   case AnalyzeJobClass: return "analyzer";
   case MigrateJobClass: return "migrator";
   case CompileJobClass: return "compiler";
-  case FortranFrontendJobClass:
-    return "fortran-frontend";
   case BackendJobClass: return "backend";
   case AssembleJobClass: return "assembler";
   case IfsMergeJobClass: return "interface-stub-merger";
@@ -404,12 +402,6 @@ void CompileJobAction::anchor() {}
 CompileJobAction::CompileJobAction(Action *Input, types::ID OutputType)
     : JobAction(CompileJobClass, Input, OutputType) {}
 
-void FortranFrontendJobAction::anchor() {}
-
-FortranFrontendJobAction::FortranFrontendJobAction(Action *Input,
-                                                   types::ID OutputType)
-    : JobAction(FortranFrontendJobClass, Input, OutputType) {}
-
 void BackendJobAction::anchor() {}
 
 BackendJobAction::BackendJobAction(Action *Input, types::ID OutputType)
