@@ -76,29 +76,6 @@ static const char *getOutputFileName(Compilation &C, StringRef Base,
   return OutputFileName;
 }
 
-static bool checkSystemForAMDGPU(const ArgList &Args, const AMDGPUToolChain &TC,
-                                 std::string &GPUArch) {
-  auto CheckError = [&](llvm::Error Err) -> bool {
-    std::string ErrMsg =
-        llvm::formatv("{0}", llvm::fmt_consume(std::move(Err)));
-    TC.getDriver().Diag(diag::err_drv_undetermined_gpu_arch)
-        << llvm::Triple::getArchTypeName(TC.getArch()) << ErrMsg << "-march";
-    return false;
-  };
-
-  auto ArchsOrErr = TC.getSystemGPUArchs(Args);
-  if (!ArchsOrErr)
-    return CheckError(ArchsOrErr.takeError());
-
-  if (ArchsOrErr->size() > 1)
-    if (!llvm::all_equal(*ArchsOrErr))
-      return CheckError(llvm::createStringError(
-          std::error_code(), "Multiple AMD GPUs found with different archs"));
-
-  GPUArch = ArchsOrErr->front();
-  return true;
-}
-
 static void addOptLevelArg(const llvm::opt::ArgList &Args,
                            llvm::opt::ArgStringList &CmdArgs, bool IsLlc) {
   StringRef OOpt = "2"; // Default if no user command line specification
@@ -128,7 +105,7 @@ static void addOptLevelArg(const llvm::opt::ArgList &Args,
       OOpt = "3";
   }
   // To remove unreferenced internalized functions, add globaldce pass to O0
-  if (OOpt.equals("0") && !IsLlc)
+  if (OOpt == "0" && !IsLlc)
     CmdArgs.push_back(Args.MakeArgString("-passes=default<O0>,globaldce"));
   else
     CmdArgs.push_back(Args.MakeArgString("-O" + OOpt));
@@ -233,7 +210,8 @@ const char *amdgpu::dlr::getLinkCommandArgs(
     llvm::opt::ArgStringList &LastLinkArgs, const ToolChain &TC,
     const llvm::Triple &Triple, llvm::StringRef TargetID,
     llvm::StringRef OutputFilePrefix, const char *InputFileName,
-    const RocmInstallationDetector &RocmInstallation) {
+    const RocmInstallationDetector &RocmInstallation,
+    llvm::opt::ArgStringList &EnvironmentLibraryPaths) {
   LastLinkArgs.push_back(Args.MakeArgString(InputFileName));
 
   // Get the environment variable ROCM_LINK_ARGS and add to llvm-link.
@@ -262,21 +240,60 @@ const char *amdgpu::dlr::getLinkCommandArgs(
 
   llvm::SmallVector<std::string, 12> BCLibs;
 
+  std::string AsanRTL;
   if (Args.hasFlag(options::OPT_fgpu_sanitize, options::OPT_fno_gpu_sanitize,
                    true) &&
       TC.getSanitizerArgs(Args).needsAsanRt()) {
-    std::string AsanRTL(RocmInstallation.getAsanRTLPath());
-    if (AsanRTL.empty()) {
-      if (!Args.hasArg(options::OPT_nogpulib))
+    if (!Args.hasArg(options::OPT_nogpulib)){
+      AsanRTL = RocmInstallation.getAsanRTLPath();
+      if(AsanRTL.empty())
         TC.getDriver().Diag(diag::err_drv_no_asan_rt_lib);
-    } else {
-      BCLibs.push_back(AsanRTL);
+      else
+        BCLibs.push_back(AsanRTL);
     }
   }
   StringRef GPUArch = getProcessorFromTargetID(Triple, TargetID);
 
-  BCLibs.push_back(
-      Args.MakeArgString(libpath + "/libomptarget-amdgpu-" + GPUArch + ".bc"));
+  // When the base lib directory is called `lib` we enable
+  // the look-up of the libomptarget bc lib to happen and if not present
+  // where it is expected it means we are using the build tree compiler
+  // not the installed compiler.
+  std::string LibDeviceName = "/libomptarget-amdgpu-" + GPUArch.str() + ".bc";
+
+  SmallString<128> Path(Args.MakeArgString(libpath + LibDeviceName));
+  if (LibSuffix != "lib" || llvm::sys::fs::exists(Path)) {
+    BCLibs.push_back(Args.MakeArgString(Path));
+  } else {
+    // Check if the device library can be found in
+    // one of the LIBRARY_PATH directories.
+    bool EnvOmpLibDeviceFound = false;
+    for (auto &EnvLibraryPath : EnvironmentLibraryPaths) {
+      std::string EnvOmpLibDevice = EnvLibraryPath + LibDeviceName;
+      if (llvm::sys::fs::exists(EnvOmpLibDevice)) {
+        EnvOmpLibDeviceFound = true;
+        BCLibs.push_back(EnvOmpLibDevice);
+        break;
+      }
+    }
+    // If LIBRARY_PATH doesn't point to the device library,
+    // then use the default one.
+    if (!EnvOmpLibDeviceFound) {
+      std::string RtDir = "/../runtimes/runtimes-bins/offload";
+      BCLibs.push_back(Args.MakeArgString(libpath + RtDir + LibDeviceName));
+    }
+  }
+
+  if (!AsanRTL.empty()) {
+    if (!Args.hasArg(options::OPT_nogpulib)) {
+      // asanrtl is dependent on ockl so for every asanrtl bitcode linking
+      // requires ockl but viceversa is not true.
+      std::string OcklRTL(RocmInstallation.getOCKLPath());
+      if (OcklRTL.empty())
+        TC.getDriver().Diag(diag::err_drv_no_rocm_device_lib);
+      else
+        BCLibs.push_back(OcklRTL);
+    }
+  }
 
   // Add the generic set of libraries, OpenMP subset only
   BCLibs.append(amdgpu::dlr::getCommonDeviceLibNames(
@@ -377,50 +394,27 @@ AMDGPUOpenMPToolChain::AMDGPUOpenMPToolChain(const Driver &D, const llvm::Triple
   getProgramPaths().push_back(getDriver().Dir);
 }
 
-AMDGPUOpenMPToolChain::AMDGPUOpenMPToolChain(const Driver &D,
-                                             const llvm::Triple &Triple,
-                                             const ToolChain &HostTC,
-                                             const ArgList &Args,
-                                             const Action::OffloadKind OK,
-                                             const std::string TargetID)
-    : ROCMToolChain(D, Triple, Args), HostTC(HostTC), OK(OK) {
-  // Lookup binaries into the driver directory, this is used to
-  // discover the clang-offload-bundler executable.
-  getProgramPaths().push_back(getDriver().Dir);
-  this->TargetID = std::move(TargetID);
-}
-
 void AMDGPUOpenMPToolChain::addClangTargetOptions(
     const llvm::opt::ArgList &DriverArgs, llvm::opt::ArgStringList &CC1Args,
     Action::OffloadKind DeviceOffloadingKind) const {
   HostTC.addClangTargetOptions(DriverArgs, CC1Args, DeviceOffloadingKind);
 
-  std::string TargetIDStr = getTargetID().str();
-  if (TargetIDStr.empty()) {
-    if (!checkSystemForAMDGPU(DriverArgs, *this, TargetIDStr))
-      return;
-  }
-  StringRef TargetID = StringRef(TargetIDStr);
-  assert((DeviceOffloadingKind == Action::OFK_HIP ||
-          DeviceOffloadingKind == Action::OFK_OpenMP) &&
-         "Only HIP offloading kinds are supported for GPUs.");
+  StringRef GPUArch = DriverArgs.getLastArgValue(options::OPT_march_EQ);
+  assert(!GPUArch.empty() && "Must have an explicit GPU arch.");
 
   CC1Args.push_back("-target-cpu");
-  StringRef GPUArch = getProcessorFromTargetID(getTriple(), TargetID);
   CC1Args.push_back(DriverArgs.MakeArgStringRef(GPUArch));
 
   // Extract all the -m options
   std::vector<llvm::StringRef> Features;
   amdgpu::getAMDGPUTargetFeatures(getDriver(), getTriple(), DriverArgs,
-                                  Features, TargetIDStr);
+                                  Features, GPUArch);
 
   for (auto OneFeature : unifyTargetFeatures(Features)) {
     CC1Args.push_back("-target-feature");
     CC1Args.push_back(OneFeature.data());
   }
 
-  CC1Args.push_back("-fcuda-is-device");
-
   if (DriverArgs.hasFlag(options::OPT_fgpu_approx_transcendentals,
                          options::OPT_fno_gpu_approx_transcendentals, false))
     CC1Args.push_back("-fcuda-approx-transcendentals");
@@ -455,6 +449,12 @@ void AMDGPUOpenMPToolChain::addClangTargetOptions(
   if (DriverArgs.hasArg(options::OPT_nogpulib))
     return;
 
+  for (auto BCFile : getDeviceLibs(DriverArgs)) {
+    CC1Args.push_back(BCFile.ShouldInternalize ? "-mlink-builtin-bitcode"
+                                               : "-mlink-bitcode-file");
+    CC1Args.push_back(DriverArgs.MakeArgString(BCFile.Path));
+  }
+
   ArgStringList LibraryPaths;
 
   // Find in --hip-device-lib-path and HIP_LIBRARY_PATH.
@@ -498,11 +498,18 @@ llvm::opt::DerivedArgList *AMDGPUOpenMPToolChain::TranslateArgs(
     }
 
     if (!DAL->hasArg(options::OPT_march_EQ)) {
-      std::string Arch = BoundArch.str();
+      StringRef Arch = BoundArch;
       if (Arch.empty()) {
-        Arch = getTargetID().str(); // arch may have come from --Offload-Arch=
-        if (Arch.empty())
-          checkSystemForAMDGPU(Args, *this, Arch);
+        auto ArchsOrErr = getSystemGPUArchs(Args);
+        if (!ArchsOrErr) {
+          std::string ErrMsg =
+              llvm::formatv("{0}", llvm::fmt_consume(ArchsOrErr.takeError()));
+          getDriver().Diag(diag::err_drv_undetermined_gpu_arch)
+              << llvm::Triple::getArchTypeName(getArch()) << ErrMsg << "-march";
+          Arch = OffloadArchToString(OffloadArch::HIPDefault);
+        } else {
+          Arch = Args.MakeArgString(ArchsOrErr->front());
+        }
       }
       DAL->AddJoinedArg(nullptr, Opts.getOption(options::OPT_march_EQ), Arch);
     }
@@ -525,6 +532,7 @@ llvm::opt::DerivedArgList *AMDGPUOpenMPToolChain::TranslateArgs(
 
 void AMDGPUOpenMPToolChain::addClangWarningOptions(
     ArgStringList &CC1Args) const {
+  AMDGPUToolChain::addClangWarningOptions(CC1Args);
   HostTC.addClangWarningOptions(CC1Args);
 }
 
@@ -547,15 +555,6 @@ void AMDGPUOpenMPToolChain::AddClangSystemIncludeArgs(const ArgList &DriverArgs,
   SmallString<128> P(HostTC.getDriver().ResourceDir);
   llvm::sys::path::append(P, "include/cuda_wrappers");
   CC1Args.push_back(DriverArgs.MakeArgString(P));
-
-  // Force APU mode will focefully include #pragma omp requires
-  // unified_shared_memory via the force_usm header
-  if (DriverArgs.hasArg(options::OPT_fopenmp_force_usm)) {
-    CC1Args.push_back("-include");
-    CC1Args.push_back(
-        DriverArgs.MakeArgString(HostTC.getDriver().ResourceDir +
-                                 "/include/openmp_wrappers/force_usm.h"));
-  }
 }
 
 /// Convert path list to Fortran frontend argument
@@ -590,7 +589,7 @@ void AMDGPUOpenMPToolChain::AddFlangSystemIncludeArgs(const ArgList &DriverArgs,
     return;
 
   {
-    SmallString<128> P(D.InstalledDir);
+    SmallString<128> P(D.Dir);
     llvm::sys::path::append(P, "../include");
     IncludePathList.push_back(DriverArgs.MakeArgString(P.str()));
   }
